jQuery(document).ready(function ($) {
    // Find the host element inserted by PHP
    const host = document.getElementById('ai-chat-shadow-host');
    if (!host) {
        console.error('Chatbox host element not found.');
        return;
    }
    
    // If the browser supports Shadow DOM, attach a shadow root.
    if (host.attachShadow) {
        const shadow = host.attachShadow({ mode: 'open' });
        const chatboxTitle = host.getAttribute('data-chatbox-title') || 'AI Chat';
        
        // Inject both the CSS and the chatbox HTML into the shadow DOM.
        shadow.innerHTML = `
            <style>
                /* Shadow DOM-scoped CSS */
                #ai-chatbox {
                    position: fixed;
                    bottom: 80px;
                    right: 20px;
                    width: 400px;
                    max-height: 500px;
                    background: white;
                    border: 1px solid #ccc;
                    border-radius: 10px;
                    box-shadow: 0px 0px 10px rgba(0, 0, 0, 0.1);
                    display: none;
                    flex-direction: column;
                    
                    /* Default font settings */
                    font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
                    font-size: 14px;
                }
                #ai-chat-header {
                    background: #244465;
                    color: white;
                    padding: 10px;
                    text-align: center;
                    font-weight: bold;
                    position: relative;
                    border-top-left-radius: 10px;
                    border-top-right-radius: 10px;
                }
                #ai-chat-close {
                    position: absolute;
                    right: 10px;
                    top: 50%;
                    transform: translateY(-50%);
                    cursor: pointer;
                }
                #ai-chat-body {
                    flex: 1;
                    padding: 10px;
                    overflow-y: auto;
                    max-height: 350px;
                    display: flex;
                    flex-direction: column;
                }
                #ai-chat-footer {
                    font-size: 9px;
                    color: #888;
                    text-align: right;
                    margin-top: 5px;
                }
                .chat-message {
                    padding: 8px;
                    margin: 5px;
                    border-radius: 10px;
                    max-width: 80%;
                    word-wrap: break-word;
                }
                .chat-message.user {
                    background: #244465;
                    color: white;
                    text-align: right;
                    align-self: flex-end;
                }
                .chat-message.bot {
                    background: #f1f1f1;
                    color: black;
                    text-align: left;
                    align-self: flex-start;
                }
                #ai-chat-input {
                    display: flex;
                    flex-direction: column;
                    padding: 10px;
                    border-top: 1px solid #ccc;
                }
                #ai-chat-input input,
                #ai-chat-input button {
                    width: 100%;
                    box-sizing: border-box;
                }
                #ai-chat-input input {
                    height: 40px;
                    padding: 5px;
                    border: 1px solid #ccc;
                    border-radius: 5px;
                    font-size: 14px;
                    margin-bottom: 5px;
                }
                #ai-chat-input button {
                    background: #244465;
                    color: white;
                    border: none;
                    padding: 8px 10px;
                    cursor: pointer;
                    border-radius: 5px;
                }
                #ai-chat-icon {
                    position: fixed;
                    bottom: 20px;
                    right: 20px;
                    background: #244465;
                    color: white;
                    padding: 15px;
                    border-radius: 50%;
                    cursor: pointer;
                    font-size: 20px;
                    text-align: center;
                    z-index: 9999;
                }
            </style>
            <div id="ai-chatbox">
                <div id="ai-chat-header">
                    ${chatboxTitle} <span id="ai-chat-close">&times;</span>
                </div>
                <div id="ai-chat-body"></div>
                <div id="ai-chat-input">
                    <input type="text" id="ai-chat-prompt" placeholder="Have any content questions?..." />
                    <button id="ai-chat-send">Send</button>
                    <div id="ai-chat-footer">
                        WordPress AI Chat by <a href="https://potentpages.com/wp-chat-ai" target="_blank">Potent Pages</a>
                    </div>
                </div>
            </div>
            <div id="ai-chat-icon">?</div>
        `;
        
        // Now select elements within the shadow DOM.
        const chatIcon = shadow.getElementById('ai-chat-icon');
        const chatBox = shadow.getElementById('ai-chatbox');
        const chatClose = shadow.getElementById('ai-chat-close');
        const chatSend = shadow.getElementById('ai-chat-send');
        const chatPrompt = shadow.getElementById('ai-chat-prompt');
        const chatBody = shadow.getElementById('ai-chat-body');
        
        let chatHistory = [];
        let initialMessagesLoaded = false;
        
        function addToChatHistory(role, message) {
            chatHistory.push({ role, message });
        }
        
        function updateChatboxState(isOpen) {
            if (isOpen) {
                chatBox.style.display = 'flex';
                if (!initialMessagesLoaded) {
                    loadInitialMessages();
                    initialMessagesLoaded = true;
                }
            } else {
                chatBox.style.display = 'none';
            }
        }
        
        // Restore state from localStorage or get the default from the server.
        const savedState = localStorage.getItem('aiChatboxState');
        if (savedState !== null) {
            updateChatboxState(savedState === 'open');
        } else {
            $.ajax({
                url: aiSearchAjax.ajax_url,
                method: 'POST',
                data: { action: 'get_default_open_state' },
                success: function (response) {
                    if (response.success) {
                        const defaultOpen = response.data.default_open_state === 'yes';
                        updateChatboxState(defaultOpen);
                        localStorage.setItem('aiChatboxState', defaultOpen ? 'open' : 'closed');
                    }
                },
                error: function () {
                    console.error('AJAX request failed.');
                }
            });
        }
        
        // Toggle chatbox visibility on icon click.
        chatIcon.addEventListener('click', function () {
            if (chatBox.style.display === 'flex') {
                chatBox.style.display = 'none';
                localStorage.setItem('aiChatboxState', 'closed');
            } else {
                chatBox.style.display = 'flex';
                localStorage.setItem('aiChatboxState', 'open');
                if (!initialMessagesLoaded) {
                    loadInitialMessages();
                    initialMessagesLoaded = true;
                }
            }
        });
        
        chatClose.addEventListener('click', function () {
            chatBox.style.display = 'none';
            localStorage.setItem('aiChatboxState', 'closed');
        });
        
        function loadInitialMessages() {
            if (chatBody.children.length > 0) return; // Avoid duplicate loading.
            $.ajax({
                url: aiSearchAjax.ajax_url,
                method: 'POST',
                data: { action: 'get_initial_messages' },
                success: function (response) {
                    if (response.success && response.data.initial_messages.length > 0) {
                        $.ajax({
                            url: aiSearchAjax.ajax_url,
                            method: 'POST',
                            data: { action: 'get_initial_message_delay' },
                            success: function (delayResponse) {
                                let delay = delayResponse.success ? parseFloat(delayResponse.data.delay) * 1000 : 500;
                                displayMessagesSequentially(response.data.initial_messages, delay);
                            },
                            error: function () {
                                displayMessagesSequentially(response.data.initial_messages, 500);
                            }
                        });
                    }
                },
                error: function () {
                    console.error('Failed to load initial messages.');
                }
            });
        }
        
        async function displayMessagesSequentially(messages, delay) {
            for (let message of messages) {
                await showMessageWithDelay(message, delay);
            }
        }
        
        function showMessageWithDelay(message, delay) {
            return new Promise((resolve) => {
                let messageDiv = document.createElement('div');
                messageDiv.classList.add('chat-message', 'bot');
                messageDiv.style.display = 'none';
                messageDiv.textContent = message;
                chatBody.appendChild(messageDiv);
                $(messageDiv).fadeIn(300, function () {
                    addToChatHistory('assistant', message);
                    setTimeout(resolve, delay);
                });
            });
        }
        
        // Send prompt on button click.
        chatSend.addEventListener('click', function () {
            var query = chatPrompt.value.trim();
            if (!query) return;
            
            // Append user message
            let userMessageDiv = document.createElement('div');
            userMessageDiv.classList.add('chat-message', 'user');
            userMessageDiv.textContent = query;
            chatBody.appendChild(userMessageDiv);
            chatPrompt.value = '';
            
            // Append placeholder for bot response.
            let botMessageDiv = document.createElement('div');
            botMessageDiv.classList.add('chat-message', 'bot');
            botMessageDiv.textContent = '...';
            chatBody.appendChild(botMessageDiv);
            
            addToChatHistory('user', query);
            chatBody.scrollTop = chatBody.scrollHeight;
            
            $.post(aiSearchAjax.ajax_url, {
                action: 'ai_search_chat',
                query: query,
                chat_history: chatHistory
            }, function (response) {
                if (response.success) {
                    const botResponse = response.data.response;
                    botMessageDiv.innerHTML = botResponse;
                    addToChatHistory('assistant', botResponse);
                } else {
                    botMessageDiv.textContent = 'Error fetching response.';
                }
                chatBody.scrollTop = chatBody.scrollHeight;
            });
        });
        
        // New: Listen for Enter key press on the input field.
        chatPrompt.addEventListener('keydown', function (e) {
            if (e.key === 'Enter') {
                e.preventDefault(); // Prevent form submission or default behavior.
                chatSend.click();   // Trigger the send button click.
            }
        });
        
    } else {
        // Fallback for browsers that do not support Shadow DOM.
        console.warn('Shadow DOM is not supported in this browser.');
        // You could optionally insert your original chatbox HTML directly here.
    }
});
